package EFS::CLI::PlatformBuild;

use strict;
use warnings;
use English;

use base q{EFS::CLI};
use base q{EFS::DBI};
use Params::Validate qw( :all );

sub define {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name     => { type => SCALAR },
        platform => { type => SCALAR },
        kernel   => { type => SCALAR },
        initrd   => { type => SCALAR },
        options  => { type => SCALAR },
    });

    my $platform_rs = $class->get_resultset( q{Platform} );

    my $platform = $platform_rs->retrieve_bynames_assert(
        name => $args{platform},
    );

    my $resultset = $class->get_resultset;

    my ($build) = $resultset->search({ name => $args{name} });

    if ( $build ) {

        foreach my $key ( qw( platform kernel initrd options ) ) {
            if ( $args{$key} eq $build->$key ) {
                STDOUT->print( qq{Verified $key value of '$args{$key}'\n} );
            } else {
                $build->set_column( $key => $args{$key} );
                $build->update;
                STDOUT->print( qq{Updated $key value to '$args{$key}'\n} );
            }
        }

    } else {
        $build = $resultset->create({
            name     => $args{name},
            platform => $platform->id,
            kernel   => $args{kernel},
            initrd   => $args{initrd},
            options  => $args{options},
        });
        STDOUT->print( qq{Defined build $args{name}\n} );
    }

    return $build;

}

sub undef {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR },
    });

    my $resultset = $class->get_resultset;

    my $build = $resultset->retrieve_bynames_assert( name => $args{name} );

    $build->delete;

    STDOUT->print( qq{Undefined build $args{name}\n} );

    return 1;

}

sub display {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR, optional => 1 },
    });

    my $resultset = $class->get_resultset;

    if ( $args{name} ) {

        my $build = $resultset->retrieve_bynames_assert(
            name => $args{name},
        );

        $class->display_objects( objects => [ $build ] );

    } else {

        $class->display_objects( objects => [ $resultset->all ] );

    }

    return 1;

}

sub history {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        platform => { type => SCALAR, optional => 1 },
        name     => { type => SCALAR },
        %EFS::CLI::History::validation_specs,
    });

    $args{platformbuild} = delete $args{name};

    return EFS::CLI::History->history(%args);

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
