package EFS::DBI::Result::Location;

use strict;
use warnings;
use English;

use base q{EFS::DBI::Result};

__PACKAGE__->load_components( qw/+EFS::DBI::Prefix Core/ );

__PACKAGE__->table( q{location} );

__PACKAGE__->add_columns( qw( campus name description ) );
__PACKAGE__->add_columns(
    id => { __PACKAGE__->get_id_column_attrs },
);
__PACKAGE__->set_primary_key( q{id} );
__PACKAGE__->sequence( q{location_seq} );

__PACKAGE__->belongs_to( campus => q{EFS::DBI::Result::Campus} );

__PACKAGE__->has_many( cells    => q{EFS::DBI::Result::Cell} );
__PACKAGE__->has_many( pods     => q{EFS::DBI::Result::Pod} );
__PACKAGE__->has_many( cabinets => q{EFS::DBI::Result::Cabinet} );

__PACKAGE__->has_many( attrs    => q{EFS::DBI::Result::LocationAttrs} );

sub region {
    return shift->campus->region;
}

sub object_id {
    my $self = shift;
    my $separator = shift || q{/};
    return join( $separator,
                 $self->region->name,
                 $self->campus->name,
                 $self->name );
}

sub domainname {
    my $self = shift;
    my $parameter_rs = $self->get_resultset( q{Parameter} );
    my $root = $parameter_rs->configured( q{root_domain} );
    return join( q{.},
                 $self->name,
                 $self->campus->name,
                 $self->region->name,
                 $root );
}

sub display_columns {
    return qw( region campus name description );
}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
