package EFS::DBI::Result::Platform;

use strict;
use warnings;
use English;

use base q{EFS::DBI::Result};

__PACKAGE__->load_components( qw/+EFS::DBI::Prefix Core/ );

__PACKAGE__->table( q{platform} );

__PACKAGE__->add_columns( qw( name backwards emulates ) );
__PACKAGE__->add_columns(
    id => { __PACKAGE__->get_id_column_attrs },
);
__PACKAGE__->set_primary_key( q{id} );
__PACKAGE__->sequence( q{platform_seq} );

__PACKAGE__->belongs_to( backwards => q{EFS::DBI::Result::Platform} );
__PACKAGE__->belongs_to( emulates => q{EFS::DBI::Result::Platform} );

__PACKAGE__->has_many( aliases => q{EFS::DBI::Result::PlatformAlias} );

__PACKAGE__->has_many( instances => q{EFS::DBI::Result::PlatformInstance} );
__PACKAGE__->has_many( builds => q{EFS::DBI::Result::PlatformBuild} );

sub forwards {
    my $self = shift;
    my $resultset = $self->get_resultset;
    my @forwards = $resultset->search({ backwards => $self->id });
    return @forwards;
}

sub emulators {
    my $self = shift;
    my $resultset = $self->get_resultset;
    my @emulators = $resultset->search({ emulates => $self->id });
    return @emulators;
}

sub installs {
    my $self            = shift;
    my $install_rs      = $self->get_resultset( q{Install} );
    my @installs        = $install_rs->search({ name => $self->name });
    return @installs;
}

sub get_alias_installs {

    my $self = shift;
    my $release = shift;

    my @aliases = $self->aliases;
    my @installs = $release->installs;

    my @results = ();

    foreach my $alias ( @aliases ) {
        foreach my $install ( @installs ) {
            if ( $install->name eq $alias->name ) {
                push @results, $install;
            }
        }
    }

    return @results;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
