package EFS::DBI::ResultSet::Fileshare;

use strict;
use warnings;
use English;

use base q{EFS::DBI::ResultSet};

use Params::Validate qw( :all );
use Readonly;

Readonly my %IS_VALID_FILETYPE => map { ( $_ => 1 ) } qw{ dev home dist };

sub validate_data {

    my $class = shift;

    my %data = @_;

    if ( exists $data{filetype}
            and not exists $IS_VALID_FILETYPE{ $data{filetype} } ) {
        die(
            qq{Invalid filetype '$data{filetype}'.  },
            q{Must be one of: },
            join( q{, }, keys %IS_VALID_FILETYPE ),
            qq{.\n},
        );
    }

    return 1;

}

sub retrieve_bynames_assert {

    my $self = shift;

    my (%args) = validate( @_, {
        cell       => { type => SCALAR },
        fileserver => { type => SCALAR },
        name       => { type => SCALAR },
    });

    my $fileserver_rs = EFS::DBI->get_resultset( q{Fileserver} );

    my $fileserver = $fileserver_rs->retrieve_bynames_assert(
        cell => $args{cell},
        name => $args{fileserver},
    );

    my ($fileshare) = $self->search({
        fileserver => $fileserver->id,
        name       => $args{name},
    });

    if ( not $fileshare ) {
        die qq{No such fileshare '$args{name}' on fileserver '$fileserver'\n};
    }

    return $fileshare;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
