package EFS::DBI::ResultSet::Region;

use strict;
use warnings;
use English;

use base q{EFS::DBI::ResultSet};

use Params::Validate qw( :all );

sub validate_data {
    my $class = shift;
    my %data = @_;
    return 1 unless exists $data{region};
    ## no critic ProhibitEnumeratedClasses
    # we really do want a-z here; [:lower:] would be effected by locale
    $data{region} =~ m/\A [a-z]+ \z/msx || die(
        qq{Invalid region '$data{region}'\n},
        qq{Must consist solely of lower case letters (a-z)\n}
    );
    return;
}

sub retrieve_bynames_assert {

    my $self = shift;

    my (%args) = validate( @_, {
        name => { type => SCALAR },
    });

    my ($region) = $self->search({ name => $args{name} });

    if ( not $region ) {
        die qq{No such region '$args{name}'\n};
    }

    return $region;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
