package EFS::Proc::Pidfile;

use strict;
use warnings;
use English;

use Moose;
use File::Slurp;

has q{pidfile} => ( is => q{ro}, isa => q{Str}, required => 1 );
has q{pid}     => ( is => q{rw}, isa => q{Int} );

sub BUILD {

    my $self = shift;

    $self->pid( $PID );

    my $pidfile = $self->pidfile;

    if ( -f $pidfile ) {
        my $pid = read_file( $pidfile );
        chomp $pid;
        if ( kill 0, $pid ) {
            die qq{$pidfile is currently in use by pid = $pid\n};
        }
    }

    write_file( $pidfile, qq{$PID\n} );

    return $self;

}

sub destroy {
    my $self = shift;
    my $pidfile = $self->pidfile;
    if ( -f $pidfile ) {
        unlink $pidfile or
            die qq{Unable to unlink pidfile $pidfile: $ERRNO\n};
    }
    return 1;
}

sub DEMOLISH {
    my $self = shift;
    return if not $PID == $self->pid;
    return $self->destroy;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
