#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 9;
use Test::Exception;

use blib;
use EFS::Proc::ProcessTable;
use EFS::Proc::Pidfile;

## no critic: RequireExtendedFormatting

{

    $ENV{FAKEPS_OUTPUT} = <<"FAKEPS_OUTPUT";
USER   PID %CPU %MEM   VSZ   RSS  TT  STAT STARTED      TIME COMMAND
root   944  0.0  0.5 20644  1172  v0  Is   11:05PM   0:00.08 login [pam] (login)
root   956  0.0  0.0  8252     0  v0  IW+  -         0:00.00 -csh (csh)
root   945  0.0  0.0  5860     0  v1  IWs+ -         0:00.00 /usr/libexec/getty Pc ttyv1
root   946  0.0  0.0  5860     0  v2  IWs+ -         0:00.00 /usr/libexec/getty Pc ttyv2
root   947  0.0  0.0  5860     0  v3  IWs+ -         0:00.00 /usr/libexec/getty Pc ttyv3
root   948  0.0  0.0  5860     0  v4  IWs+ -         0:00.00 /usr/libexec/getty Pc ttyv4
root   949  0.0  0.0  5860     0  v5  IWs+ -         0:00.00 /usr/libexec/getty Pc ttyv5
root   950  0.0  0.0  5860     0  v6  IWs+ -         0:00.00 /usr/libexec/getty Pc ttyv6
root   951  0.0  0.0  5860     0  v7  IWs+ -         0:00.00 /usr/libexec/getty Pc ttyv7
root  2960  0.0  0.8  9188  1920   0  Ss   11:28PM   0:00.32 bash
root 74568  0.0  0.5  6976  1260   0  R+    9:54PM   0:00.00 ps -u
root 63802  0.0  0.4 10276  1080   1  Is    1:07AM   0:00.02 -csh (csh)
root 63812  0.0  0.5 20636  1232   1  I     1:08AM   0:00.01 su - efstest
FAKEPS_OUTPUT

    my $table = EFS::Proc::ProcessTable->new(
        command => q{./test/bin/fakeps},
    );

    ok( ref $table && $table->isa( q{EFS::Proc::ProcessTable} ),
        q{EFS::Proc::ProcessTable->new returns the right object} );

    my $processes = $table->table;

    ok( ref $processes eq q{ARRAY}, q{table returns ARRAYREF} );

    ok( scalar @{$processes} == 13, q{Correct process count} );

    my $process = $processes->[0];

    ok( ref $process && $process->isa( q{EFS::Proc::Process} ),
        q{EFS::Proc::Process object type} );

    ok( $process->pid == 944,       q{Correct pid} );
    ok( $process->pctcpu eq q{0.0}, q{Correct pctcpu} );
    ok( $process->pctmem eq q{0.5}, q{Correct pctmem} );
    ok( $process->size == 20644,    q{Correct size} );
    ok( $process->rss == 1172,      q{Correct rss} );

}

my $pidfile = qq{/var/tmp/EFS_Proc_tests_$PID};

1;

