#!/usr/bin/env perl
use strict;
use warnings;
use English;

use blib;
use EFS;

use lib q{test/lib};
use EFS::Test;
use EFS::Test::Config;

use Test::More tests => 3;

my $test   = EFS::Test->new;
my $config = $test->get_config;

my $ldc_efsserver = $config->get_value( q{LocalDevCell}, q{efsserver} );

if ( not $config->get_value( q{Security}, q{kerberos_realm} ) ) {
    plan skip_all => qq{Test suite not configured for GSSAPI\n};
}

# Try to connect with PAM -- should fail
$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs showcreds efsserver -serverauth PAM},
    not     => 1,
    expect  => [
        q{This server does not support authtype 'PAM'},
        q{Exit status from efs server not received. Assuming failure...},
    ],
);

# Try to connect with GSSAPI, no ticket -- should fail
# The actual strings are unfortunately version dependent
$test->efs_test_command(
    run_as  => q{efsops},
    command => q{KRB5CCNAME=FILE:/dev/null efs showcreds efsserver},
    not     => 1,
    expect  => [
        q{GSSAPI Authentication Failure:},
        q{Unable to initialize output context},
        qr{GSS \s+ Generic \s+ Error:}msx,
        qr{GSS \s+ Specific \s+ Error:}msx,
        q{This server does not support authtype 'PAM'},
        q{Exit status from efs server not received. Assuming failure...},
    ],
);

# Try to connect with GSSAPI, with ticket -- should work
$test->efs_test_command(
    run_as  => q{efsops},
    command => q{EFS_CLIENT_DEBUG=1 efs showcreds efsserver},
    filter  => qr{\[DEBUG\]}msx,
    expect  => [
        qr{Connected \s+ to \s+ $ldc_efsserver:\d+}msx,
        qr{Authenticated \s+ via \s+ GSSAPI \s+ with \s+ efsd\@$ldc_efsserver}msx,
        qr{Received \s+ status \s+ message: \s+ _EFS_EXIT_STATUS: \s+ 0}msx,
    ],
);

1;

