#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 113;

use lib q{test/lib};
use EFS::Test;

## no critic: RequireExtendedFormatting

my $test = EFS::Test->new( as_user => q{primary_user} );

my $m_name   = q{testmeta};
my $mp_name  = q{testmeta testproj};
my $mpr_name = q{testmeta testproj testrel};
my $mp_path  = q{testmeta/testproj};
my $mpr_path = q{testmeta/testproj/testrel};

my @setup = (
    q{efs display metaproj testmeta},
    q{efs display project testmeta testproj},
    q{efs create release testmeta testproj testrel},
);

my @extensions = qw( build test runtime
                     circle-a circle-b circle-c
                     another-a another-b );

foreach my $extension (@extensions) {
    push @setup, qq{efs create release $mpr_name-$extension };
}

foreach my $setup (@setup) {
    $test->efs_test_command(
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

$test->efs_test_command(
    command => qq{efs create releasealias $mpr_name-rlink testrel-runtime},
    expect  => [
        qq{Created symlink /efs/dev/$mpr_path-rlink -> testrel-runtime},
        qq{Created releasealias $mpr_path-rlink -> testrel-runtime},
    ],
);


$test->efs_test_command(
    command => qq{efs define dependency $mpr_name $mpr_name-build build},
    expect  => qq{Defined dependency of $mpr_path on $mpr_path-build, type build},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name $mpr_name-test test},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name $mpr_name-runtime runtime},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name $mpr_name-rlink runtime},
);

foreach my $kind ( qw( dependency dependents ) ) {

    foreach my $name ( $mpr_name, $mp_name, $m_name ) {

        next if $name eq $mpr_name and $kind eq q{dependents};

        $test->efs_test_command(
            command => qq{efs display $kind $name},
            expect  => [
                qq{Dependent Release Type},
                qr{=+}ms,
                qq{$mpr_path $mpr_path-build build},
                qq{$mpr_path $mpr_path-runtime runtime},
                qq{$mpr_path $mpr_path-test test},
                qq{Dependent ReleaseAlias Type},
                qr{=+}ms,
                qq{$mpr_path $mpr_path-rlink runtime},
            ],
        );

        $test->efs_test_command(
            command => qq{efs display $kind $name --type runtime},
            expect  => [
                qq{Dependent Release Type},
                qr{=+}ms,
                qq{$mpr_path $mpr_path-runtime runtime},
                qq{Dependent ReleaseAlias Type},
                qr{=+}ms,
                qq{$mpr_path $mpr_path-rlink runtime},
            ],
        );

        $test->efs_test_command(
            command => qq{efs display $kind $name --type build},
            expect  => [
                qq{Dependent Release Type},
                qr{=+}ms,
                qq{$mpr_path $mpr_path-build build},
            ],
        );

        $test->efs_test_command(
            command => qq{efs display $kind $name --type test},
            expect  => [
                qq{Dependent Release Type},
                qr{=+}ms,
                qq{$mpr_path $mpr_path-test test},
            ],
        );

    }

}

$test->efs_test_command(
    command => qq{efs display dependents $mpr_name-runtime},
    expect  => [
        qq{Dependent Release Type},
        qr{=+}ms,
        qq{$mpr_path $mpr_path-runtime runtime},
        qq{Dependent ReleaseAlias Type},
        qr{=+}ms,
        qq{$mpr_path $mpr_path-rlink runtime},
    ],
);

$test->efs_test_command(
    command => qq{efs display dependents $mpr_name-runtime --direct},
    expect  => [
        qq{Dependent Release Type},
        qr{=+}ms,
        qq{$mpr_path $mpr_path-runtime runtime},
    ],
);

$test->efs_test_command(
    command => qq{efs display dependents $mpr_name-build},
    expect  => [
        qq{Dependent Release Type},
        qr{=+}ms,
        qq{$mpr_path $mpr_path-build build},
    ],
);

$test->efs_test_command(
    command => qq{efs display dependents $mpr_name-test},
    expect  => [
        qq{Dependent Release Type},
        qr{=+}ms,
        qq{$mpr_path $mpr_path-test test},
    ],
);

$test->efs_test_command(
    command => qq{efs display dependents $mpr_name-rlink},
    expect  => [
        qq{Dependent ReleaseAlias Type},
        qr{=+}ms,
        qq{$mpr_path $mpr_path-rlink runtime},
    ],
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name $mpr_name-build build},
    expect  => qq{Undefined dependency of $mpr_path on $mpr_path-build},
    bail    => 1,
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name $mpr_name-test test},
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name $mpr_name-runtime runtime},
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name $mpr_name-rlink runtime},
    bail    => 1,
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name nosuch nosuch nosuch runtime},
    not     => 1,
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name $mpr_name-runtime nosuch},
    not     => 1,
);

$test->efs_test_command(
    command => qq{efs batchdefine dependency $mpr_name runtime $mpr_path-runtime,$mpr_path-rlink},
    expect  => [
        qq{Defined runtime dependency of $mpr_path on $mpr_path-rlink},
        qq{Defined runtime dependency of $mpr_path on $mpr_path-runtime},
        q{All dependencies verified or defined as specified.},
    ],
);

$test->efs_test_command(
    command => qq{efs batchundef dependency $mpr_name runtime $mpr_path-runtime,$mpr_path-rlink},
    expect  => [
        qq{Undefined runtime dependency of $mpr_path on $mpr_path-runtime},
        qq{Undefined runtime dependency of $mpr_path on $mpr_path-rlink},
        q{All specified dependencies verified as missing or undefined.},
    ],
);

$test->efs_test_command(
    command => qq{efs batchdefine dependency $mpr_name runtime $mpr_path-runtime,nometa/noproj/norel},
    not     => 1,
    expect  => [
        q{Invalid dependencies specified.},
        q{Must be releases or releasealiases:},
        q{nometa/noproj/norel},
    ],
);

# Test basic circular dependency check in define dependency

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-a $mpr_name-circle-b runtime},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-b $mpr_name-circle-c runtime},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-c $mpr_name-circle-a runtime},
    not     => 1,
    expect  => [
        qq{The dependent ($mpr_path-circle-c) is one of the dependencies of $mpr_path-circle-a:},
        qr{Stage Depth Diststate Runtime Dependencies}ms,
        qr{^=+}ms,
        qr{dev 1 nocp $mpr_path-circle-a}ms,
        qr{dev 2 nocp $mpr_path-circle-b}ms,
        qr{dev 3 nocp $mpr_path-circle-c \(circular\)}ms,
        q{This would create a circular dependency.},
        q{Invalid dependency specification.},
    ],
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name-circle-b $mpr_name-circle-c runtime},
);

# Test circular dependency check in batchdefine dependency

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-a $mpr_name-circle-c runtime},
);

$test->efs_test_command(
    command =>
        qq{efs batchdefine dependency $mpr_name-circle-c runtime } .
            qq{$mpr_path-circle-a,$mpr_path-circle-b},
    not    => 1,
    expect => [
        q{Invalid dependencies specified.},
        q{Circular dependencies are not allowed.},
        qq{Dependent '$mpr_path-circle-c' is a dependency of:},
        qr{Stage Depth Diststate Runtime Dependencies}ms,
        qr{^=+}ms,
        qr{dev 1 nocp $mpr_path-circle-a}ms,
        qr{dev 2 nocp $mpr_path-circle-b}ms,
        qr{dev 2 nocp $mpr_path-circle-c \(circular\)}ms,
    ],
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name-circle-a $mpr_name-circle-c runtime},
);

# Test the showtree commands

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-a $mpr_name-circle-b runtime},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-b $mpr_name-circle-c runtime},
);

$test->efs_test_command(
    command => qq{efs showtree dependency $mpr_name-circle-a runtime},
    expect  => [
        qr{Stage Depth Diststate Runtime Dependencies}ms,
        qr{^=+}ms,
        qr{dev 1 nocp $mpr_path-circle-a}ms,
        qr{dev 2 nocp $mpr_path-circle-b}ms,
        qr{dev 3 nocp $mpr_path-circle-c}ms,
    ],
);

$test->efs_test_command(
    command => qq{efs showtree dependency $mpr_name-circle-a runtime -conflicts nosuch},
    not     => 1,
    expect  => q{No such metaproj 'nosuch'},
);

$test->efs_test_command(
    command => qq{efs showtree dependency $mpr_name-circle-a runtime -conflicts testmeta/nosuch},
    not     => 1,
    expect  => q{No such project 'testmeta/nosuch'},
);

$test->efs_test_command(
    command => qq{efs showtree dependency $mpr_name-circle-a runtime -conflicts testmeta/testproj/invalid},
    not     => 1,
    expect  => [
        q{Invalid conflicts specification.},
        q{Only metaprojs or projects may be specified.},
        q{The value 'testmeta/testproj/invalid' is invalid.},
    ],
);

foreach my $conflict ( q{all}, q{testmeta}, q{testmeta/testproj} ) {
    $test->efs_test_command(
        command => qq{efs showtree dependency $mpr_name-circle-a runtime }
            . qq{-conflicts $conflict},
        expect => [
            qr{Stage Depth Diststate Runtime Dependencies}ms,
            qr{^=+}ms,
            qr{dev 1 nocp $mpr_path-circle-a \(conflict\)}ms,
            qr{dev 2 nocp $mpr_path-circle-b \(conflict\)}ms,
            qr{dev 3 nocp $mpr_path-circle-c \(conflict\)}ms,
        ],
    );
}

$test->efs_test_command(
    command => qq{efs showtree dependents $mpr_name-circle-c runtime},
    expect  => [
        qr{Stage Depth Diststate Runtime Dependents}ms,
        qr{^=+}ms,
        qr{dev 1 nocp $mpr_path-circle-c}ms,
        qr{dev 2 nocp $mpr_path-circle-b}ms,
        qr{dev 3 nocp $mpr_path-circle-a}ms,
    ],
);

$test->efs_test_command(
    command => qq{efs showtree dependency $mpr_name-circle-a runtime -maxdepth 2},
    expect  => [
        qr{Stage Depth Diststate Runtime Dependencies}ms,
        qr{^=+}ms,
        qr{dev 1 nocp $mpr_path-circle-a}ms,
        qr{dev 2 nocp $mpr_path-circle-b}ms,
    ],
);

$test->efs_test_command(
    command => qq{efs showtree dependents $mpr_name-circle-c runtime -maxdepth 2},
    expect  => [
        qr{Stage Depth Diststate Runtime Dependents}ms,
        qr{^=+}ms,
        qr{dev 1 nocp $mpr_path-circle-c}ms,
        qr{dev 2 nocp $mpr_path-circle-b}ms,
    ],
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-another-a $mpr_name-another-b runtime},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-b $mpr_name-another-a runtime},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-c $mpr_name-another-a runtime},
);

$test->efs_test_command(
    command => qq{efs showtree dependency $mpr_name-circle-a},
    expect  => [
        qr{Stage Depth Diststate Runtime Dependencies}ms,
        qr{^=+}ms,
        qr{dev 1 nocp $mpr_path-circle-a}ms,
        qr{dev 2 nocp $mpr_path-circle-b}ms,
        qr{dev 3 nocp $mpr_path-another-a \(repeat\)}ms,
        qr{dev 3 nocp $mpr_path-circle-c}ms,
        qr{dev 4 nocp $mpr_path-another-a}ms,
        qr{dev 5 nocp $mpr_path-another-b}ms,
    ],
);

# Cleanup some of the dependencies, so we can redefine them for the
# releasealias tests

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name-another-a $mpr_name-another-b runtime},
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name-circle-b $mpr_name-another-a runtime},
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name-circle-c $mpr_name-another-a runtime},
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name-circle-a $mpr_name-circle-b runtime},
);

$test->efs_test_command(
    command => qq{efs undef dependency $mpr_name-circle-b $mpr_name-circle-c runtime},
);

# Test the create releasealias sanity check

$test->efs_test_command(
    command => qq{efs create releasealias $mpr_name-circle testrel-circle-b},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-c $mpr_name-circle-a runtime},
);

$test->efs_test_command(
    command => qq{efs define dependency $mpr_name-circle-a $mpr_name-circle runtime},
);

$test->efs_test_command(
    command => qq{efs create releasealias $mpr_name-circle testrel-circle-c},
    not     => 1,
    expect  => [
        q{This would create a circular runtime dependency:},
        qr{Stage Depth Diststate Runtime Dependencies}ms,
        qr{^=+}ms,
        qr{dev 1 nocp $mpr_path-circle-c}ms,
        qr{dev 2 nocp $mpr_path-circle-a}ms,
        qr{dev 3 \S+ $mpr_path-circle \(circular\)}ms,
        qr{dev 4 nocp $mpr_path-circle-b}ms,
        q{Invalid releasealias target.},
    ],
);

# Teardown

my @teardown = (
    qq{efs undef dependency $mpr_name-circle-c $mpr_name-circle-a runtime},
    qq{efs undef dependency $mpr_name-circle-a $mpr_name-circle runtime},
    qq{efs destroy releasealias $mpr_name-circle},
    qq{efs destroy releasealias $mpr_name-rlink},
    qq{efs setstage release $mpr_name rejected},
    qq{efs destroy release $mpr_name},
    qq{efs purge release $mpr_name -now},
);

foreach my $extension (@extensions) {
    push @teardown, (
        qq{efs setstage release $mpr_name-$extension rejected},
        qq{efs destroy release $mpr_name-$extension},
        qq{efs purge release $mpr_name-$extension -now},
    );
}

foreach my $teardown (@teardown) {
    $test->efs_test_command(
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

exit 0;
