#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 16;

use lib q{test/lib};
use EFS::Test;

my $test = EFS::Test->new;

my @setup = (
    q{efs display metaproj testmeta},
    q{efs display project testmeta testproj},
    q{efs create release testmeta testproj incr},
    q{efs create install testmeta testproj incr common},
    q{touch /efs/dev/testmeta/testproj/incr/install/common/foo},
    q{efs checkpoint release testmeta testproj incr},
    q{efs dist release testmeta testproj incr},
);

foreach my $setup (@setup) {
    $test->efs_test_command(
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs display feature allow_insecure_prod_incr},
    expect  => q{No such feature 'allow_insecure_prod_incr'},
    not     => 1,
    bail    => 1,
);

# By default, no unlocked incr releases can be prod

$test->efs_test_command(
    command => q{efs setstage release testmeta testproj incr prod},
    not     => 1,
    expect  => [
        q{Validating the checkpoint state for release 'testmeta/testproj/incr'.},
        q{Validating the audit state for release 'testmeta/testproj/incr'.},
        q{Unable to set the stage to prod, because the release is not locked.},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs define feature allow_insecure_prod_incr 1},
    expect  => [ q{Defined feature 'allow_insecure_prod_incr'}, ],
);

$test->efs_test_command(
    command => q{efs setstage release testmeta testproj incr prod},
    expect  => [
        q{Validating the checkpoint state for release 'testmeta/testproj/incr'.},
        q{Validating the audit state for release 'testmeta/testproj/incr'.},
        q{Setting the unlocked release testmeta/testproj/incr stage to prod,},
        q{since the allow_insecure_prod_incr feature is enabled.},
        q{Changed the stage of testmeta/testproj/incr from dev to prod},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs undef feature allow_insecure_prod_incr},
    expect  => [ q{Undefined feature 'allow_insecure_prod_incr'}, ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs history feature allow_insecure_prod_incr -count 2},
    expect  => [
        qr{History \s+ ID \s+ StartTime \s+ Duration \s+ Status \s+ Authuser \s+ Request}xms,
        qr{-+}xms,
        qr{define \s+ feature \s+ allow_insecure_prod_incr}xms,
        qr{undef \s+ feature \s+ allow_insecure_prod_incr}xms,
    ],
);

my @teardown = (
    q{efs setstage release testmeta testproj incr deprecated},
    q{efs destroy release testmeta testproj incr},
    q{efs purge release testmeta testproj incr -now},
);

foreach my $teardown (@teardown) {
    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

exit 0;

