#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More qw( no_plan );

use lib q{test/lib};
use EFS::Test;

my $test = EFS::Test->new( as_user => q{efsops} );

my $region = q{testregion};
my $campus = q{testcampus};

my $noiger = reverse $region;

$test->efs_test_command( command => qq{efs define region $region 'test region'}, );

$test->efs_test_command(
    name    => q{Verifying that the campus doesn't exist...},
    command => qq{efs display campus $region $campus},
    not     => 1,
    bail    => 1,
    expect  => [ qq{No such campus $region/$campus}, ],
);

$test->efs_test_command(
    name    => q{Define a bogus campus},
    command => qq{efs define campus $region UPPERCASE 'test campus'},
    not     => 1,
    expect  => [
        q{Invalid campus 'UPPERCASE'},
        q{Must consist solely of lower case letters (a-z)},
        q{Exit status from efs server not received.  Assuming failure...},
    ],
);

$test->efs_test_command(
    name    => qq{Defined the campus '$campus'},
    command => qq{efs define campus $region $campus 'temp campus'},
    expect  => [ qq{Defined campus $region/$campus}, ],
);

$test->efs_test_command(
    name    => qq{Defined the campus '$campus' again},
    command => qq{efs define campus $region $campus 'test campus'},
    expect  => [
        qq{Campus $region/$campus is already defined},
        qq{Updated description of campus $region/$campus},
    ],
);

$test->efs_test_command(
    name    => q{Define the other region},
    command => qq{efs define region $noiger 'other region'},
);

$test->efs_test_command(
    name    => q{Try to define the campus in the other region},
    command => qq{efs define campus $noiger $campus 'test campus'},
    not     => 1,
    expect =>
        [ qq{Campus $campus is already defined, but in region $region}, ],
);

$test->efs_test_command(
    name    => q{Set an attribute for the campus},
    command => ( qq{efs setattr campus $region $campus foo bar}, ),
    expect  => [
        qq{For campus '$region/$campus', defined attribute as:},
        q{'foo' => 'bar'},
    ],

);

$test->efs_test_command(
    command => qq{efs display region $region},
    expect  => [
        qr{Region \s+ Description}msx,
        qr{=+}msx,
        qr{$region \s+ test \s+ region}msx,
        qr{Region \s+ Campus \s+ Description}msx,
        qr{=+}msx,
        qr{$region \s+ $campus \s+ test \s campus}msx,
    ],
);

$test->efs_test_command(
    command => qq{efs display campus $region $campus},
    expect  => [
        qr{Region \s+ Campus \s+ Description}msx,
        qr{=+}msx,
        qr{$region \s+ $campus \s+ test \s+ campus}msx,
        qr{Region \s+ Campus \s+ Attribute \s+ Value}msx,
        qr{=+}msx,
        qr{$region \s+ $campus \s+ foo \s+ bar}msx,
    ],
);

$test->efs_test_command(
    name    => q{Try to undef the region prematurely},
    command => qq{efs undef region $region},
    not     => 1,
    expect  => [
        qq{Unable to undef region $region, because it has the},
        q{following campuses:}, qq{$campus},
    ],
);

$test->efs_test_command(
    command => qq{efs undef campus $region $campus},
    expect  => [ qq{Undefined campus $region/$campus}, ],
);

$test->efs_test_command( command => qq{efs undef region $region}, );

$test->efs_test_command( command => qq{efs undef region $noiger}, );

$test->efs_test_command(
    command => qq{efs history campus $region $campus -count 4},
    expect  => [
        qr{History \s+ ID \s+ StartTime \s+ Duration \s+ Status \s+ Authuser \s+ Request}msx,
        qr{-+}msx,
        qr{define \s+ campus \s+ $region \s+ $campus}msx,
        qr{define \s+ campus \s+ $region \s+ $campus}msx,
        qr{setattr \s+ campus \s+ $region \s+ $campus \s+ foo \s+ bar}msx,
        qr{undef \s+ campus \s+ $region \s+ $campus}msx,
    ],
);


